package org.diagbot.common.push.cache;

import org.diagbot.common.push.bean.ResultMappingFilter;
import org.diagbot.common.push.bean.Rule;
import org.diagbot.common.push.bean.RuleApp;
import org.diagbot.nlp.participle.cfg.Configuration;
import org.diagbot.nlp.participle.cfg.DefaultConfig;
import org.diagbot.nlp.util.NlpCache;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class ApplicationCacheUtil {

    //词库同义词定义
    public static Map<String, Map<String, String>> standard_info_synonym_map = null;
    //诊断科室衍射
    public static Map<String, String> doc_result_mapping_diag_map = null;
    //特征性别 年龄过滤等
    public static Map<String, Map<String, ResultMappingFilter>> doc_result_mapping_filter_map = null;
    //化验辅检体征 性别 年龄过滤等
    public static Map<String, Map<String, ResultMappingFilter>> doc_result_mapping_lpvSex_filter_map = null;
    // 规则
    public static Map<String, List<Rule>> kl_rule_filter_map = null;
    //危险值提醒
    public static Map<String, RuleApp> kl_rule_app_filter_map = null;
    //pacs关系抽取过滤
    public static Map<String, Map<String, String>> kl_diagnose_detail_filter_map = null;

    public static Map<String, Map<String, String>> getStandard_info_synonym_map() {
        if (standard_info_synonym_map == null) {
            standard_info_synonym_map = NlpCache.getStandard_info_synonym_map();
        }
        return standard_info_synonym_map;
    }

    public static Map<String, String> getDoc_result_mapping_diag_map() {
        if (doc_result_mapping_diag_map == null) {
            createDoc_result_mapping_diag_map();
        }
        return doc_result_mapping_diag_map;
    }

    public static Map<String, String> createDoc_result_mapping_diag_map() {
        Configuration configuration = new DefaultConfig();
        doc_result_mapping_diag_map = configuration.loadMapDict("bigdata_diag_2_dept.dict");
        return doc_result_mapping_diag_map;
    }

    public static Map<String, Map<String, ResultMappingFilter>> getDoc_result_mapping_lpvSex_filter_map() {
        if (doc_result_mapping_lpvSex_filter_map == null) {
            createDoc_result_mapping_lpvSex_filter_map();
        }
        return doc_result_mapping_lpvSex_filter_map;
    }

    public static Map<String, Map<String, ResultMappingFilter>> getDoc_result_mapping_filter_map() {
        if (doc_result_mapping_filter_map == null) {
            createDoc_result_mapping_filter_map();
        }
        return doc_result_mapping_filter_map;
    }

    public static Map<String, Map<String, ResultMappingFilter>> createDoc_result_mapping_filter_map() {
        Configuration configuration = new DefaultConfig();
        List<String> fileContents = configuration.readFileContents("bigdata_sex_age_filter.dict");
        String[] line_string;
        List<ResultMappingFilter> resultMappingFilters = new ArrayList<>();
        try {
            for (int i = 0; i < fileContents.size(); i++) {
                line_string = org.apache.commons.lang3.StringUtils.split(fileContents.get(i), "\\|");
                if (line_string.length == 5) {
                    ResultMappingFilter resultMappingFilter = new ResultMappingFilter();
                    resultMappingFilter.setFeatureName(line_string[0]);
                    resultMappingFilter.setFeatureType(line_string[1]);
                    resultMappingFilter.setSex(line_string[2]);
                    resultMappingFilter.setAgeStart(Integer.parseInt(line_string[3]));
                    resultMappingFilter.setAgeEnd(Integer.parseInt(line_string[4]));
                    resultMappingFilters.add(resultMappingFilter);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        doc_result_mapping_filter_map = new HashMap<>();
        Map<String, ResultMappingFilter> filterMap = null;
        for (ResultMappingFilter resultMappingFilter : resultMappingFilters) {
            filterMap = doc_result_mapping_filter_map.get(resultMappingFilter.getFeatureType());
            if (filterMap == null) {
                filterMap = new HashMap<>();
            }
            filterMap.put(resultMappingFilter.getFeatureName(), resultMappingFilter);
            doc_result_mapping_filter_map.put(resultMappingFilter.getFeatureType(), filterMap);
        }
        return doc_result_mapping_filter_map;
    }

    /**
     * 化验辅检体征年龄性别过滤
     *
     * @return
     */
    public static Map<String, Map<String, ResultMappingFilter>> createDoc_result_mapping_lpvSex_filter_map() {
        Configuration configuration = new DefaultConfig();
        List<String> fileContents = configuration.readFileContents("bigdata_lpv_sex_age_filter.dict");
        String[] line_string;
        List<ResultMappingFilter> resultMappingFilters = new ArrayList<>();
        try {
            for (int i = 0; i < fileContents.size(); i++) {
                line_string = org.apache.commons.lang3.StringUtils.split(fileContents.get(i), "\\|");
                if (line_string.length == 5) {
                    ResultMappingFilter resultMappingFilter = new ResultMappingFilter();
                    resultMappingFilter.setFeatureName(line_string[0]);
                    resultMappingFilter.setFeatureType(line_string[1]);
                    resultMappingFilter.setSex(line_string[2]);
                    resultMappingFilter.setAgeStart(Integer.parseInt(line_string[3]));
                    resultMappingFilter.setAgeEnd(Integer.parseInt(line_string[4]));
                    resultMappingFilters.add(resultMappingFilter);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        doc_result_mapping_lpvSex_filter_map = new HashMap<>();
        Map<String, ResultMappingFilter> filterMap = null;
        for (ResultMappingFilter resultMappingFilter : resultMappingFilters) {
            filterMap = doc_result_mapping_lpvSex_filter_map.get(resultMappingFilter.getFeatureType());
            if (filterMap == null) {
                filterMap = new HashMap<>();
            }
            filterMap.put(resultMappingFilter.getFeatureName(), resultMappingFilter);
            doc_result_mapping_lpvSex_filter_map.put(resultMappingFilter.getFeatureType(), filterMap);
        }
        return doc_result_mapping_lpvSex_filter_map;
    }


    public static Map<String, List<Rule>> getKl_rule_filter_map() {
        if (kl_rule_filter_map == null) {
            create_kl_rule_filter_map();
        }
        return kl_rule_filter_map;
    }

    public static void create_kl_rule_filter_map() {
        kl_rule_filter_map = new HashMap<>();

        Configuration configuration = new DefaultConfig();
        List<String> fileContents = configuration.readFileContents("bigdata_rule_filter.dict");

        List<Rule> rules = null;
        for (String line : fileContents) {
            String[] content = line.split("\\|", -1);
            Rule rule = new Rule();
            if (content.length == 12) {
                rule.setId(content[0] == null ? "" : content[0]);
                rule.setPub_name(content[1] == null ? "" : content[1]);
                rule.setMin_operator(content[2] == null ? "" : content[2]);
                rule.setMin_value(content[3] == null ? "" : content[3]);
                rule.setMin_unit(content[4] == null ? "" : content[4]);
                rule.setMax_operator(content[5] == null ? "" : content[5]);
                rule.setMax_value(content[6] == null ? "" : content[6]);
                rule.setMax_unit(content[7] == null ? "" : content[7]);
                rule.setEq_operator(content[8] == null ? "" : content[8]);
                rule.setEq_value(content[9] == null ? "" : content[9]);
                rule.setEq_unit(content[10] == null ? "" : content[10]);
                rule.setRemind(content[11] == null ? "" : content[11]);
                if (kl_rule_filter_map.get(rule.getPub_name()) == null) {
                    rules = new ArrayList<>();
                } else {
                    rules = kl_rule_filter_map.get(rule.getPub_name());
                }
                rules.add(rule);
                kl_rule_filter_map.put(rule.getPub_name(), rules);
            }
        }
    }

    public static Map<String, RuleApp> getKl_rule_app_filter_map() {
        if (kl_rule_app_filter_map == null) {
            create_kl_rule_app_filter_map();
        }
        return kl_rule_app_filter_map;
    }

    public static void create_kl_rule_app_filter_map() {
        kl_rule_app_filter_map = new HashMap<>();

        Configuration configuration = new DefaultConfig();
        List<String> fileContents = configuration.readFileContents("bigdata_rule_app_filter.dict");

        for (String line : fileContents) {
            String[] content = line.split("\\|", -1);
            RuleApp ruleApp = new RuleApp();
            if (content.length == 4) {
                ruleApp.setId(content[0] == null ? "" : content[0]);
                ruleApp.setRuleIds(content[1] == null ? "" : content[1]);
                ruleApp.setTypeId(content[2] == null ? "" : content[2]);
                ruleApp.setRemind(content[3] == null ? "" : content[3]);
                kl_rule_app_filter_map.put(ruleApp.getId(), ruleApp);
            }
        }
    }

    public static Map<String, Map<String, String>> getKl_diagnose_detail_filter_map() {
        if (kl_diagnose_detail_filter_map == null) {
            create_kl_diagnose_detail_filter_map();
        }
        return kl_diagnose_detail_filter_map;
    }


    public static void create_kl_diagnose_detail_filter_map() {
        kl_diagnose_detail_filter_map = new HashMap<>();
        Map<String, String> diagnoseDetailRelationMap = new HashMap<>();
        Set<String> diagnoseDetailRelation = new HashSet<>();
        Configuration configuration = new DefaultConfig();
        List<String> fileContents = configuration.readFileContents("bigdata_diagnose_detail_filter.dict");
        for (String line : fileContents) {
            String[] content = line.split("\\|", -1);
            String[] relations = content[1].split("、");
            for (String relation : relations) {
                if (diagnoseDetailRelation.add(relation)) {
                    if (kl_diagnose_detail_filter_map.get(content[0]) == null) {
                        diagnoseDetailRelationMap.put(relation, relation);
                        kl_diagnose_detail_filter_map.put(content[0], diagnoseDetailRelationMap);
                    } else {
                        kl_diagnose_detail_filter_map.get(content[0]).put(relation, relation);
                    }
                }
            }
        }
    }
}
